;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x7927ca83f129a0a94f166bc61de4b1ad . "./code.cg2js/cg-status-bar.fasl")
(#x4b968a812be3765b36079390a7314b01 . "./code.cg2js/cg-hotspot.fasl")
(#x1bf8d81b42e4393b330b03995bef26d0 . "./code.cg2js/cg-toolbar.fasl")
(#xa3c67cea1d5e2a0fef7290abce8c09e7 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x4c1d33e5169da8fa4b85744ee08407da . "./code.cg2js/cg-tooltip.fasl")
(#x9d3b75d6b6d67f0a4f429de28f53797c . "./code.cg2js/cg-calendar.fasl")
(#xef908db796096e9c050814ce1f84126b . "./code.cg2js/cg-item-list.fasl")
(#xca2871b85b25206a5ab16b79582720c9 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xc83a2fa63c5531dcf31af8179dcc3bcd . "./code.cg2js/cg-trackbar.fasl")
(#x77b1430f71a1c46ef0fdcdf70c2812e3 . "./code.cg2js/cg-class-grid.fasl")
(#xa8cd4129e5baf72cf352e22d06f1aa21 . "./code.cg2js/cg-dialog-item.fasl")
(#xab91535b9a5cea00803f289cc4676030 . "./code.cg2js/cg-comtab.fasl")
(#x4b9ddba84d3f4e89a3ec1c7f4c390287 . "./code.cg2js/cg-lisp-text.fasl")
(#xd5c2525d0c16fbad42fc59c7d6266079 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x94bde61ce91d3c392f39457ecab35582 . "./code.cg2js/cg-button.fasl")
(#xe0ed73112d093bf46dc2fedd34881581 . "./code.cg2js/cg-printing.fasl")
(#xfeaecb3ac16aa102023d460122244612 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x6bedfb84687093e6320714275cc606af . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xb4ed861babc9493f073abc7a471607fd . "./code.cg2js/cg-pixmap-widget.fasl")
(#xba0df325be7ea9de9a46da557449c32a . "./code.cg2js/cg-selected-object.fasl")
(#x05c9408d88e471eb8aed5853f99535fc . "./code.cg2js/cg-text-or-combo.fasl")
(#xa9a3edc0975cd28dafa7e66d4ba751b4 . "./code.cg2js/cg-graphics-context.fasl")
(#x80f7e667895fac9fce1f689b10662f91 . "./code.cg2js/cg-edit-in-place.fasl")
(#x31cf403f086149a034b181f4760747db . "./code.cg2js/cg-string-dialog.fasl")
(#xf5cead889598bc5ec08c0be0a09f3de2 . "./code.cg2js/cg-list-view.fasl")
(#x0e1a894eb4e654b0715cfbbda3ad7380 . "./code.cg2js/cg-intersect.fasl")
(#x3399fd9ff87c3dffbe964ef4f87434f0 . "./code.cg2js/cg-text-widget.fasl")
(#x4bd1d1077799a6715d5c6d78c8a503a9 . "./code.cg2js/cg-directory-dialog.fasl")
(#x3cc457a8c0a40654dda088a2968d7853 . "./code.cg2js/cg-file-dialog.fasl")
(#xaa25f2af4376465995a14deea6f63a64 . "./code.cg2js/cg-dde-utils.fasl")
(#x800b532088ebf67ae7ab2a0146d83db1 . "./code.cg2js/cg-nodes.fasl")
(#x9e08f5fe460718c4464c666dd10a3744 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xde8a3ba6f144a5328fec4ddf93c98c00 . "./code.cg2js/cg-up-down-control.fasl")
(#x8d1988cb3c2d1c73f8eb62782e49a861 . "./code.cg2js/cg-get-position.fasl")
(#x9d88d5bcd62199b2ff0d1b4fbf255685 . "./code.cg2js/cg-toggling-widget.fasl")
(#xbe518ce65b27b2e82f63b380ba213430 . "./code.cg2js/cg-acache.fasl")
(#x64ba711844cda43d62e71e156006a0f1 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xf6fd653b7a9afbcc67815713a8429ca2 . "./code.cg2js/cg-base.fasl")
(#xb3c5bea168b0c26025f3dda000687937 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x4aa037beea805959be99d105922b37d4 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x8bb4ef85544206cdb462026d8851eb68 . "./code.cg2js/cg-property.fasl")
(#x02773a2b7b057b80c2efff696adc668a . "./code.cg2js/cg-editable-text.fasl")
(#x2cbc06580234d2c32da8c93a7ef2384f . "./code.cg2js/cg-lamp.fasl")
(#xc2c31ceb8e941d3a071464c39b12d214 . "./code.cg2js/cg-progress-indicator.fasl")
(#x48f7da33616cf009caa127376fe04816 . "./code.cg2js/cg-chart-widget.fasl")
(#x9259ae813fdb882b56217f976c364ab3 . "./code.cg2js/cg-common-control.fasl")
(#x95f6f9d717a7076e2f4ffbd6e9432b6b . "./code.cg2js/cg-font-dialog.fasl")
(#xd6559f53f320a35f0834579e93c33a6f . "./code.cg2js/cg-gesture-emulation.fasl")
(#xe673b9173a926074017b9d906b1670c3 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x345532754dc38363cf7824d6e1a2cf6d . "./code.cg2js/cg-os-window.fasl")
(#x9f1074ef0a43015405325b2908da0403 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x0d424bd2b026f017b202e0f69278ff20 . "./code.cg2js/cg-pixmap.fasl")
(#x0c58cec0236567b8cdeaef8f387e0d3a . "./code.cg2js/cg-chart-or-plot.fasl")
(#x18bcf0822b94078849732fd8b52d8f9c . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xe4ac23d753eff1d1bb9775a84fa9f93e . "./code.cg2js/cg-scaling-stream.fasl")
(#x59c795680d1dcdd5c9165d1d7914fe71 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xd1134c35436efe4b923feb08ef9eff1b . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x8fa5117aa0598f79a201579e90a999d1 . "./code.cg2js/cg-timer.fasl")
(#x7736d017bb005b5732321fbfa5999513 . "./code.cg2js/cg-scroll-bar.fasl")
(#x7eaa7c34208ecef5dc6cf083c1854de6 . "./code.cg2js/cg-caret.fasl")
(#x5702f0c857bf894847912bc1f66320ae . "./code.cg2js/cg-template-string.fasl")
(#x6d250467a3bd867f28564e1e602a217f . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xfa53ef3b0655fbcac787d3f3b3f1d6d7 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xc2bb5b1d79cb35ade922b107ea6ba379 . "./code.cg2js/cg-rich-edit.fasl")
(#x3014c89fe11b746a25d7639d7d4b3f49 . "./code.cg2js/cg-combo-box.fasl")
(#xe2ed47828af11cd0efe6c9fa0579f494 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x76be1b0eed1cc059e0350bc8f86316be . "./code.cg2js/cg-multi-picture-button.fasl")
(#x5e3128a268ceddcb31af50ce862da04e . "./code.cg2js/cg-menu-tooltip.fasl")
(#x9014b137076e0016b4e72e4f5ab9ca83 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x3f4a2324ba0d84b9c62ca57ea6bdbdac . "./code.cg2js/cg-outline.fasl")
(#x50e14a32e873c5428baf994ae02171cc . "./code.cg2js/cg-class-support.fasl")
(#x9fb9708b6871a6071ca2b1ea572ef2ce . "./code.cg2js/cg-header-control.fasl")
(#x38392b776b97ae0f2113e74427367041 . "./code.cg2js/cg-lettered-menu.fasl")
(#xbeddf6109df09f0575a00088fc40096f . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x4466a133ab8f10a2056ff0f83ef58eed . "./code.cg2js/cg-fill-texture.fasl")
(#xe2da20e820e5b02ffb669e853ec66ab8 . "./code.cg2js/cg-lisp-widget.fasl")
(#xaa667ff0bce42496b717d83c76b444a2 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xa850039308f6a4bc341718ce6c8ed0af . "./code.cg2js/cg-clipboard.fasl")
(#x622bb8b50f53302f8c13f941dc8000d5 . "./code.cg2js/cg-wrap-string.fasl")
(#xddae15fd9f42fbe23ab3cfa8e0259fcf . "./code.cg2js/cg-os-widget.fasl")
(#x300cac5959819cf5218ed801608049cf . "./code.cg2js/cg-check-box.fasl")
(#x989b1f099407e08f62361501e5dacfd4 . "./code.cg2js/cg-choice-list.fasl")
(#x4974967cccdbe86952295c706797d274 . "./code.cg2js/cg-layout.fasl")
(#x2d6e5bf728d38377cc10f5d4e84b8cce . "./code.cg2js/cg-group-box.fasl")
(#xb4620e33b3d56ace29c9ff90b185c336 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x31391c135849347103243a8b86a7f1c2 . "./code.cg2js/cg-radio-button.fasl")
(#x77c88f7745ce7fd3db387ab6dc25214f . "./code.cg2js/cg-tab-control.fasl")
(#x46d8215869ceeff1693cc0c594734b20 . "./code.cg2js/cg.fasl")
(#xf8e878b43a2d01ffbfdb818a146deec5 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x8a8afb3bfa30097c6d2fce87267954ce . "./code.cg2js/cg-dropping-outline.fasl")
(#x3b8a0c73ec12825ad79424aadd050674 . "./code.cg2js/cg-get-pixmap.fasl")
(#x013fe667fea2feea1ccfa6cbf55cdf4a . "./code.cg2js/cg-grid-widget.fasl")
(#x0b0b7da608234e43cd1beff73f434f2c . "./code.cg2js/cg-utility-dialog.fasl")
(#xcc503d08d76cbe0653534b060d42ae59 . "./code.cg2js/cg-static-text.fasl")
(#x3a867713d27fb6579d86ead266d005b0 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x1da316bd4fc7a04618aa94c6384c400e . "./code.cg2js/cg-object-editor-layout.fasl")
(#xa0ab5f9283b8558502bfa0ffdf42f5e1 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x3917086441d260f0d3b77c6d4324165f . "./code.cg2js/cg-bitmap-pane.fasl")
(#x450577fba63b60a6bc9123f01d65303c . "./code.cg2js/cg-plot-widget.fasl")
(#xabc0e92212fc725ddb640dd2c66464a4 . "./code.cg2js/cg-color-dialog.fasl")
(#x85ccc865964908f78eea0ed307e82705 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xdedfdd79fc32accfb9c8ecfc18879a9f . "./code.cg2js/cg-icon.fasl")
(#x66534cd47a932ca3f547c26a148225ac . "./code.cg2js/cg-dev.fasl")
(#x05c5ec0d4edd214bfcaa1ab400a206a9 . "./code.cg2js/cg-choose-printer.fasl")
(#x2b26a5d4759cb781cba97eda973520ba . "./code.cg2js/cg-intersect-posbox.fasl")
(#x398f61e6e424025909628335c1d43b09 . "./code.cg2js/cg-menu.fasl")
(#xf65a9f8acefff40c3611773144ba4fb7 . "./code.cg2js/cg-yes-no-list.fasl")
(#x3ed3ab8ff9b135a8c542fec5b6886310 . "./code.cg2js/cg-palette.fasl")
(#x08a33381c1d489c3d18746cfe3cb8e39 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xabeac36bde32b0b2828a784bc3abf8bd . "./code.cg2js/cg-message-dialog.fasl")
(#x9388871f465221291d3c0391c8f23508 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x7c101501e51384245a36bff0c8730080 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x07c4d5ef9da045c2899c83818b26c7b9 . "./code.cg2js/cg-split-bar.fasl")
(#x6cf40a08bce8d2b1d9b2c1ebf0fcf58d . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x02a2de03aee8fe77bba4efba443521fe . "./code.cg2js/cg-web-browser.fasl")
(#x715c5d039932570e5370c0e955a65ecf . "./code.cg2js/cg-project-window.fasl")
(#x82f993048f053a84f493175d560b1460 . "./code.cg2js/cg-picture-widget.fasl")
(#x9e551fc38f1c2fc1f9f314b185137194 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xf9153eb36f59ce66adc3fa9465c6c465 . "./code.cg2js/cg-object-editor.fasl")
(#x9b2ae7a2797ee2e7f86a0d48d71d20c3 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x83ac8b3f13096e9212b7060e154788e0 . "./code.cg2js/cg-paren-matching.fasl")
(#x20bd6ea6b1e9e19b0a9f2af7311b6b63 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xea8e667ed7b4bcd9294e591b3c509976 . "./code.cg2js/cg-drawable.fasl")
(#x4cf1301ca8308b5c3e4ec9057928a859 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x967ca89683fedaa8a8721d6d8d24e800 . "./code.cg2js/cg-html-widget.fasl")
(#x6b7ac45fe1cb67e1c856ab6eb1d61877 . "./code.cg2js/cg-curve.fasl")
(#x888c7b1d1c2fe7549f45a5b7350f27d2 . "./code.cg2js/cg-yes-no-string.fasl")
(#xf25925cebc7cfc459cc292609ecb2d20 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x811258d55ca0407031bf7ef4a2ad4ff0 . "./code.cg2js/cg-html-dialog.fasl")
(#x39abfd88a733bba6a33a7a0c7dd6d655 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xe46e63642efa649c070a1e1a57dfe7d6 . "./code.cg2js/cg-tray.fasl")
(#x00c32cbede6347549c1f89c67957c7cc . "./code.idejs/ide-apropos.fasl")
(#x64ae91162a91ebe09a5d5139e2824d1d . "./code.idejs/ide-window-list.fasl")
(#x66b2128ecc632654b9ee6d2692557194 . "./code.idejs/ide-class-browser.fasl")
(#x6337dc532543f8a31ea95e689f6ab9d8 . "./code.idejs/ide-base.fasl")
(#x793a2c9d8d4562379225b782a767a0f4 . "./code.idejs/ide-process-browser.fasl")
(#x315be868c0dbdb839083dec499e5a98b . "./code.idejs/ide-acache.fasl")
(#xeb67aa2ad224d44381ba2b1c0a075998 . "./code.idejs/ide-navigator.fasl")
(#x71f6a9e15647aba623f46c9bae392142 . "./code.idejs/ide-trace.fasl")
(#xba44e8c43d631fadcef702970fb320d5 . "./code.idejs/ide-project.fasl")
(#xbf4a26c4a16a1f8c0c6bde44d4485441 . "./code.idejs/ide-patch-dialog.fasl")
(#x4e26e3e17abc8565ba70d1114624f18c . "./code.idejs/ide-grapher.fasl")
(#x2656058b95b035903dbf457cf0f9fe05 . "./code.idejs/ide-dev.fasl")
(#x0627e25ae89d5e1761082aecd2792f3a . "./code.idejs/ide-keyshort.fasl")
(#x0abc14b6959984b30081adee048ddf66 . "./code.idejs/ide-editor.fasl")
(#x94a9206c9f2f716db432bf742d47ac96 . "./code.idejs/ide-inspector.fasl")
(#x29017a945852d5daebf5fd2185674a82 . "./code.idejs/ide.fasl")
(#xa1e2ef58fe88d1b2d9b2c2be18210cc1 . "./code.idejs/ide-clipboard.fasl")
(#x602049cda3cd5ba5128881c41422e23c . "./code.idejs/ide-find-definitions.fasl")
(#x51ea8df2a645ed6ce76ddbdf07856ac2 . "./code.idejs/ide-form.fasl")
(#x92103d1ff3d335d2626164cdda57e29b . "./code.idejs/ide-intro.fasl")
(#x3e681968e254feafeece519f1a261a75 . "./code.idejs/ide-find-in-files.fasl")
(#x1a1a220eede61a17adcb62509f162235 . "./code.idejs/ide-package-list.fasl")
(#x0a7a1bb2db9a79f8172ca07fd43648c5 . "./code.idejs/ide-class-grapher.fasl")
(#xe396744af4d34e05e734175c54f8cafc . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
